package	jp.co.sds_corp.util.pooh.frame;

import java.awt.Rectangle;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.simple.SimpleInternalFrame;
import jp.co.sds_corp.util.pooh.top.Toplevel;

public class FrameManager {
	Map frameMap = new HashMap();
	Toplevel toplevel = null;

	public FrameManager(Toplevel _toplevel) {toplevel = _toplevel;}
	public void remove(String fn) { frameMap.remove(fn); }
	public void removeAll() { frameMap.clear();}

	public String makeKey(SimpleInternalFrame sif) {
		return sif.getFrame_id() 
				+ "@"
				+ sif.getClass().getName()
				+ "@"
				+ Integer.toHexString(sif.hashCode());
	}

	public void add(String fn, Object frame) {
		SimpleInternalFrame sif = ((FrameResource)frame).getFrame();
		if (sif != null) {
			frameMap.put(makeKey(sif), frame);
		} else {
			frameMap.put(fn, frame);
		}
	}

	public Object get(String fn) { return frameMap.get(fn); } 
	public Object getFrameResource(String fn) throws Exception {
		FrameParser parser = new FrameParser(toplevel);
		return parser.loadResource(fn);
	}

	public Object [] getFramesByKey(String fn) throws Exception {
		List list = new ArrayList();
		Iterator iterator = frameMap.keySet().iterator();
		String full_key = "";
		while (iterator.hasNext()) {
			String key = (String)iterator.next();
			if (fn.equals(key.substring(0, key.indexOf("@")))) {
				list.add(frameMap.get(key));
			}
		}
		FrameResource [] result = new FrameResource[list.size()];
		return (FrameResource [])list.toArray(result);
	}

	public Object [] getFramesByKey(String fn, String title) throws Exception {
		List list = new ArrayList();
		Iterator iterator = frameMap.keySet().iterator();
		String full_key = "";
		while (iterator.hasNext()) {
			String key = (String)iterator.next();
			if (fn.equals(key.substring(0, key.indexOf("@")))) {
				FrameResource fr = (FrameResource)frameMap.get(key);
				if (fr.getFrame().getTitle().equals(title)) {
					list.add(frameMap.get(key));
				}
			}
		}
		FrameResource [] result = new FrameResource[list.size()];
		return (FrameResource [])list.toArray(result);
	}

	public Object [] getAllFrames() throws Exception {
		List list = new ArrayList();
		Iterator iterator = frameMap.keySet().iterator();
		String full_key = "";
		while (iterator.hasNext()) {
			String key = (String)iterator.next();
			list.add(frameMap.get(key));
		}
		FrameResource [] result = new FrameResource[list.size()];
		return (FrameResource [])list.toArray(result);
	}

	public Iterator iterator() { return frameMap.values().iterator(); }

	public void loadResourceAll(String directory, String file, String resType) { 
		try {
			FrameParser parser = new FrameParser(toplevel);
			FrameResource [] frs = parser.loadResourceAll(directory, file, resType);
			for (int i=0; i<frs.length; i++)
				add(frs[i].getFrame_id(), frs[i]);
			return;
		} catch (Exception e) {}
	}

	public Object createFrame(FrameResource request) throws Exception {
		request.setFrame(new SimpleInternalFrame(toplevel, request));
		add(makeKey(request.getFrame()), request);
		return request;
	}

	public Object createFrame(String fn) throws Exception {
		FrameParser parser = new FrameParser(toplevel);
		return createFrame(parser.loadResource(fn));
	}

	public boolean isFrame(String fn) {
		boolean alive = false;
		Iterator iterator = frameMap.keySet().iterator();	
		while (iterator.hasNext()) {
			String key = (String)iterator.next();
			if (fn.equals(key.substring(0, key.indexOf ("@")))) {
				alive = true;
				break;
			}
		}
		return alive;
	}

	public boolean isAlive(String fn) { return (get(fn) != null) ? true : false; }
	public Object createFrame(String fn, int open) throws Exception {
		if (open == SubMenuResource.OPEN_SINGLE) {
			if (isFrame(fn)) {
				return null;
			}
			FrameParser parser  = new FrameParser(toplevel);
			FrameResource res   = parser.loadResource(fn);
			return createFrame(res);
		} else {
			FrameParser parser  = new FrameParser(toplevel);
			FrameResource res   = parser.loadResource(fn);
			int i = frameMap.size();
			res.setFrame_name(fn + ":" + (i+1));
			return createFrame(res);
		} 
	}

	public void closed() {
		Properties prop = new Properties();
		Iterator iterator = frameMap.values().iterator();
		int i = 0;
		while (iterator.hasNext()) {
			FrameResource info = (FrameResource)iterator.next();
			String id = info.getFrame_id();

			SimpleInternalFrame frame = info.getFrame();
			Rectangle rect = frame.getBounds();

			prop.put("FRAME." + i + "." + id + ".X", String.valueOf(rect.getX()));
			prop.put("FRAME." + i + "." + id + ".Y", String.valueOf(rect.getY()));
			prop.put("FRAME." + i + "." + id + ".W", String.valueOf(rect.getWidth()));
			prop.put("FRAME." + i + "." + id + ".H", String.valueOf(rect.getHeight()));
			i++;
			frame.closed();
		}
		frameMap.clear();

		try {
			FileOutputStream os = new FileOutputStream("screen.properties");
			prop.store(os, (String)toplevel.getAttribute(Toplevel.PRODUCT));
			os.flush();
			os.close();
		} catch (Exception e) { e.printStackTrace(); }
	}
}
