package	jp.co.sds_corp.util.pooh.frame;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.pooh.menu.PopupParser;
import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.top.Toplevel;
import jp.co.sds_corp.util.prop.CmnPropParser;
import jp.co.sds_corp.util.prop.ELM;
import jp.co.sds_corp.util.xml.CmnXMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
public class FrameParser {
	private Toplevel toplevel;
	public FrameParser(Toplevel _toplevel) { toplevel = _toplevel; }
	public FrameResource loadResource(String fn) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn);
			return parser.getFrameResource();
		} else {
			LoadProperty parser = new LoadProperty(fn);
			return parser.getFrameResource();
		}
	}

	public FrameResource loadResource(String fn, String path) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getFrameResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getFrameResource();
		}
	}

	public FrameResource loadResource(String fn, String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getFrameResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getFrameResource();
		}
	}

	public FrameResource [] loadResourceAll(String directory, String file, String resType) 
				throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(null, directory, file);
			return parser.getFrameResources();
		} else {
			LoadProperty parser = new LoadProperty(null, directory, file);
			return parser.getFrameResources();
		}
	}

	private class LoadProperty extends CmnPropParser {
		private FrameResource frameResource = null;
		private List list = new ArrayList();
		private String fn;
		private String path;

		public LoadProperty(String _fn) throws Exception {
			fn = _fn;
			path = null;
			parse(parseFile(toplevel.getResourcePath() + ".FRAME-INF.frame"));
		}

		public LoadProperty(String _fn, String _path) throws Exception {
			fn = _fn;
			path = null;
			parse(parseFile(_path));
		}

		public LoadProperty(String _fn, String _directory, String _file) throws Exception {
			fn = _fn;
			path = _directory;
			parse(parseFile(_directory + "/" + _file));
		}

		public void parse(ELM root) throws Exception {
			if (root == null) {
				throw new Exception("not tag <frames>");
			} else {
				boolean match = false;
				ELM [] e = getSpecificELMsByELM(root, "frames");
				if (e.length <= 0)
					return;
				ELM [] c = getSpecificELMsByELM(e[0], "frame");
				for (int i=0; i<c.length; i++) {
					String id = getAttributeByELM(c[i], "FrameID");
					if (fn != null) {
						if (id.equals(fn)) {
							frameResource = new FrameResource();
							frameResource.setFrame_id(id);
							frameDetail(c[i]);
							list.add(frameResource);
							return;
						}
					} else {
						frameResource = new FrameResource();
						frameResource.setFrame_id(id);
						frameDetail(c[i]);
						list.add(frameResource);
						match = true;
					}
				}
				if (!match) { throw new Exception("<FrameID> : keys not found." + fn); }
			}
		}
		public FrameResource getFrameResource() { return frameResource; }
		public FrameResource [] getFrameResources() { 
			FrameResource [] result = new FrameResource[list.size()];
			return (FrameResource [])list.toArray(result);
		}
		private void frameDetail(ELM e) throws Exception {
			frameResource.setFrame_name(getAttributeByELM(e, "FrameName"));
			frameResource.setFrame_type(getAttributeByELM(e, "FrameType"));
			frameResource.setFrame_x(getAttributeByELM(e, "FrameX"));
			frameResource.setFrame_y(getAttributeByELM(e, "FrameY"));
			frameResource.setFrame_w(getAttributeByELM(e, "FrameSizeW"));
			frameResource.setFrame_h(getAttributeByELM(e, "FrameSizeH"));
			frameResource.setFrame_c(getAttributeByELM(e, "FrameClosable"));
			frameResource.setFrame_m(getAttributeByELM(e, "FrameMaximizable"));
			frameResource.setFrame_r(getAttributeByELM(e, "FrameResizable"));
			frameResource.setFrame_i(getAttributeByELM(e, "FrameIconifiable"));

			frameResource.setFrame_p(getAttributeByELM(e, "Panel"));
			frameResource.setFrame_popup(getAttributeByELM(e, "Popup"));
			frameResource.setFrame_popup_menu(getAttributeByELM(e, "Popupmenu"));
			if (frameResource.getFrame_popup())
				frameResource.setKeyMap(popupDetail(frameResource.getFrame_popup_menu()));
		}

		private RootMenuResource popupDetail(String parent) throws Exception {
			PopupParser parser = new PopupParser(toplevel);
			if (path != null) {
				return parser.loadResource(parent, path + "/" + "popup.xml", "properties");
			} else {
				return parser.loadResource(parent);
			}
		}
	}

	private class LoadXML extends CmnXMLParser {
		private FrameResource frameResource = null;
		private List list = new ArrayList();
		private String fn;
		private String path = null;

		public LoadXML(String _fn) throws Exception {
			path = null;
			fn = _fn;
			URL url = toplevel.getResource("FRAME-INF.frame", "xml");
			String charset = toplevel.getPoohCharset();
			parse(parse(url.openStream()));
		}

		public LoadXML(String _fn, String _path) throws Exception {
			fn = _fn;
			path = null;
			parse(parseFile(new File(_path)));
		}

		public LoadXML(String _fn, String _directory, String _file) throws Exception {
			fn = _fn;
			path = _directory;
			parse(parseFile(new File(_directory, _file)));
		}

		public void parse(Document doc) throws Exception {
			if (doc == null) {
				throw new Exception("not tag <frames>");
			} else {
				boolean match = false;
				Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "frames");
				if (e.length <= 0)
					return;
				Element [] c = getSpecificElementsByNode(e[0], "frame");
				for (int i=0; i<c.length; i++) {
					String id = getAttributeByElement(c[i], "FrameID");
					if (fn != null) {
						if (id.equals(fn)) {
							frameResource = new FrameResource();
							frameResource.setFrame_id(id);
							frameDetail(c[i]);
							list.add(frameResource);
							return;
						}
					} else {
						frameResource = new FrameResource();
						frameResource.setFrame_id(id);
						frameDetail(c[i]);
						list.add(frameResource);
						match = true;
					}
				}
				if (!match) { throw new Exception("<FrameID> : keys not found." + fn); }
			}
		}

		public FrameResource getFrameResource() { return frameResource; }
		public FrameResource [] getFrameResources() { 
			FrameResource [] result = new FrameResource[list.size()];
			return (FrameResource [])list.toArray(result);
		}
		private void frameDetail(Element e) throws Exception {
			frameResource.setFrame_name(getAttributeByElement(e, "FrameName"));
			frameResource.setFrame_type(getAttributeByElement(e, "FrameType"));
			frameResource.setFrame_x(getAttributeByElement(e, "FrameX"));
			frameResource.setFrame_y(getAttributeByElement(e, "FrameY"));
			frameResource.setFrame_w(getAttributeByElement(e, "FrameSizeW"));
			frameResource.setFrame_h(getAttributeByElement(e, "FrameSizeH"));
			frameResource.setFrame_c(getAttributeByElement(e, "FrameClosed"));
			frameResource.setFrame_p(getAttributeByElement(e, "Panel"));
			frameResource.setFrame_popup(getAttributeByElement(e, "Popup"));
			frameResource.setFrame_popup_menu(getAttributeByElement(e, "Popupmenu"));
			if (frameResource.getFrame_popup())
				frameResource.setKeyMap(popupDetail(frameResource.getFrame_popup_menu()));
		}
		private RootMenuResource popupDetail(String parent) throws Exception {
			PopupParser parser = new PopupParser(toplevel);
			if (path != null) {
				return parser.loadResource(parent, path + "/" + "popup.xml", "xml");
			} else {
				return parser.loadResource(parent);
			}
		}
	}
}
