package	jp.co.sds_corp.visualpooh;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.simple.SimpleTree;
import jp.co.sds_corp.util.window.SizedIcon;
public class PopupResourceTree extends JPanel {

	private DefaultMutableTreeNode toplevelNode;
	private NodeInfo toplevelFolder = null;
	private SimpleTree treeView = null;

	public PopupResourceTree() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createLoweredBevelBorder());
	}
	public String getName() { return "MenuBar View"; }
	public void createTree(VisualPoohDoc doc, RootMenuResource _res) {
		ResourceTreeCellRenderer rend = new ResourceTreeCellRenderer();

		/* Toplevel tree */
		toplevelFolder = new NodeInfo(0, 0, 0, _res);
		toplevelNode = new DefaultMutableTreeNode(toplevelFolder);
		toplevelFolder.setIcon("/resource/menu.gif");
		createMenuTree(_res);

		treeView = new SimpleTree(toplevelNode);
		treeView.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		treeView.setCellRenderer(rend);

		ResourceTreeModel model = new ResourceTreeModel(toplevelNode);
		treeView.setModel(model);

		JScrollPane scroller = new JScrollPane();
		JViewport port = scroller.getViewport();
		port.add(treeView);
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add("Center", scroller);
		add("Center", panel);
		treeView.expandAll();
	}

	private void createMenuTree(RootMenuResource _res) {
		subMenus(toplevelNode, _res, _res.getName());
	}

	private void subMenus(DefaultMutableTreeNode _node, RootMenuResource _root, String key) {
		NodeInfo folder = null;
		DefaultMutableTreeNode node = null;
		Iterator iterator = _root.iterator(key);
		if (iterator != null) {
			while (iterator.hasNext()) {
				SubMenuResource info = (SubMenuResource)iterator.next();
				switch (info.getType()) {
					case SubMenuResource.MENU_NORMAL:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/normal.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						_node.add(node);
						break;

					case SubMenuResource.MENU_SEPARATOR:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/separator.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						_node.add(node);
						break;

					case SubMenuResource.MENU_MENU:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/menu.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						subMenus(node, _root, info.getName());
						_node.add(node);
						break;

					case SubMenuResource.MENU_GROUP:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/group.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						subMenus(node, _root, info.getName());
						_node.add(node);
						break;

					case SubMenuResource.MENU_CHECK:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/checkbox.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						_node.add(node);
						break;

					case SubMenuResource.MENU_RADIO:
						folder = new NodeInfo(info.getType(), 2, 0, info);
						folder.setIcon("/resource/radio.gif");
						folder.setNode(_node);
						node = new DefaultMutableTreeNode(folder);
						_node.add(node);
						break;
				}
			}
		}
	}

	public void removeLeaf(DefaultMutableTreeNode node) {
		NodeInfo info = (NodeInfo)node.getUserObject();
		if (info.getLevel() == 2) {
			SubMenuResource smr = (SubMenuResource)info.getInfo();
			RootMenuResource rmr = smr.getRootMenuResource();
			rmr.remove(smr.getParent_name(), smr);
		} else {
		}
		((ResourceTreeModel)treeView.getModel()).removeNodeFromParent(node);
	}

	public DefaultMutableTreeNode addLeaf(int nType, int nLevel, Object obj) {
		String _name;
		String _icon;
		int _leaf;
		DefaultMutableTreeNode _node = null;
		NodeInfo info = null;
		switch (nLevel) {
			case 1:
				RootMenuResource rmr = (RootMenuResource)obj;
				_leaf = 0;
				_icon = "/resource/menu.gif";
				info = new NodeInfo(nType, nLevel, _leaf, obj);
				info.setIcon(_icon);
				info.setNode(toplevelNode);
				_node = new DefaultMutableTreeNode(info);
				break;	

			case 2:
				SubMenuResource smr = (SubMenuResource)obj;
				switch (nType) {
					case SubMenuResource.MENU_NORMAL:
						info = new NodeInfo(nType, nLevel, 1, smr);
						info.setIcon("/resource/normal.gif");
						break;
					case SubMenuResource.MENU_MENU:
						info = new NodeInfo(nType, nLevel, 0, smr);
						info.setIcon("/resource/menu.gif");
						break;
					case SubMenuResource.MENU_GROUP:
						info = new NodeInfo(nType, nLevel, 0, smr);
						info.setIcon("/resource/group.gif");
						break;
					case SubMenuResource.MENU_SEPARATOR:
						info = new NodeInfo(nType, nLevel, 1, smr);
						info.setIcon("/resource/separator.gif");
						break;
					case SubMenuResource.MENU_CHECK:
						info = new NodeInfo(nType, nLevel, 1, smr);
						info.setIcon("/resource/checkbox.gif");
						break;
					case SubMenuResource.MENU_RADIO:
						info = new NodeInfo(nType, nLevel, 1, smr);
						info.setIcon("/resource/radio.gif");
						break;
				}
				_node = new DefaultMutableTreeNode(info);
				break;	
		}
		return _node;
	}

	public void reload() {
		((ResourceTreeModel)treeView.getModel()).reload();
		treeView.expandAll();
	}

	public class NodeInfo {
		int level;	// 0:toplevel 1:rootmenu 2:submenu
		int type;	// ���֥�˥塼�Υ�����
		int leaf;

		SizedIcon icon  = null;;
		String path     = null;
		String fileName = null;
		Object info     = null;
		Object parent   = null;
		Object node     = null;

		public NodeInfo(int _type, int _level, int _leaf, Object _info) {
			level = _level;
			type = _type;
			leaf = _leaf;
			info = _info;
		}

		public SizedIcon toIcon() { return icon; }
		public String toString() { return getName(); }
		public int getLevel() { return level; }
		public int getType() { return type; }
		public int getLeaf() { return leaf; }
		public void setParent(Object _parent) { parent = _parent; }
		public void setNode(Object _node) { node = _node; }
		public void setIcon(String _icon) {
			java.net.URL img = getClass().getResource(_icon);
			icon = new SizedIcon(img, 16);
		}

		public void changeIcon(int type) {
			switch (type) {
				case SubMenuResource.MENU_NORMAL:
					setIcon("/resource/normal.gif");
					break;
				case SubMenuResource.MENU_MENU:
					setIcon("/resource/menu.gif");
					break;
				case SubMenuResource.MENU_RADIO:
					setIcon("/resource/radio.gif");
					break;
				case SubMenuResource.MENU_CHECK:
					setIcon("/resource/checkbox.gif");
					break;
				case SubMenuResource.MENU_SEPARATOR:
					setIcon("/resource/separator.gif");
					break;
				case SubMenuResource.MENU_GROUP:
					setIcon("/resource/group.gif");
					break;
			}
		}

		public Object getNode() { return node; }
		public Object getInfo() { return info; }
		public Object getParentInfo() { return parent; }
		public String getName() {
			switch (getLevel()) {
				case 0:
				case 1: return ((RootMenuResource)info).getName();
				case 2: 
					if (((SubMenuResource)info).getType()==SubMenuResource.MENU_SEPARATOR)
						return "���ѥ졼��";
					else if (type==SubMenuResource.MENU_GROUP)
						return ((SubMenuResource)info).getName();
					else
						return ((SubMenuResource)info).getName();
				default: return "";
			}
		}
	}

	public class ResourceTreeModel extends DefaultTreeModel {
		public boolean isLeaf(Object obj) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)obj;
			NodeInfo info = (NodeInfo)node.getUserObject();
			switch (info.getType()) {
				case SubMenuResource.MENU_NORMAL:
				case SubMenuResource.MENU_RADIO:
				case SubMenuResource.MENU_CHECK:
				case SubMenuResource.MENU_SEPARATOR: return true;
				default: return false;
			}
		}

		public void removeNodeFromParent(MutableTreeNode node) {
			super.removeNodeFromParent(node);
		}

		ResourceTreeModel(TreeNode node) {
			super(node);
		}
	}

	public class ResourceTreeCellRenderer extends DefaultTreeCellRenderer {
		public Component getTreeCellRendererComponent(JTree tree,
								Object value,
								boolean selected,
								boolean expanded,
								boolean leaf,
								int row,
								boolean hasFocus) {
			Component c = super.getTreeCellRendererComponent(tree, value, selected, expanded,
											leaf, row, hasFocus);

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			NodeInfo nodeInfo = (NodeInfo)node.getUserObject();
			setBackgroundNonSelectionColor(tree.getBackground());

			SizedIcon icon = nodeInfo.toIcon();
			if (icon != null)  {
				setIcon(icon);
				return this;
			}
			return c;
		}
	}
	public JTree getTreeView() { return treeView; }
	public String getToplevelTreeName() { return toplevelFolder.toString(); }
	public DefaultMutableTreeNode getToplevelTreeNode() { return toplevelNode; }
}
